#include "stdafx.h"

#define SCREEN_WIDTH 864
#define SCREEN_HEIGHT 480

#define SCENE_DURATION 200

#define TIME_INTERVAL_MS 10

int current_time = 0;

const GLfloat transitionDistance = 0.95f;

typedef struct {
	GLfloat x;
	GLfloat y;
	GLfloat z;
} Displacement;

typedef struct {
	GLfloat angle;
	GLfloat x;
	GLfloat y;
	GLfloat z;
} Rotation;

void PerformLeftHandTransitionAndRotation() 
{
	Displacement displacement;
	Rotation rotation;
	ZeroMemory(&displacement, sizeof(Displacement));
	ZeroMemory(&rotation, sizeof(Rotation));

	GLfloat progress = (GLfloat)current_time / SCENE_DURATION;
	GLfloat progress_normalized;

	if (progress < 0.1f) {
		progress_normalized = 0;
	}
	else if(progress < 0.65f)
	{
		progress_normalized = (progress - 0.1f) / 0.55f;
	}
	else if(progress >= 0.65f)
	{
		progress_normalized = 1;
	}

	displacement.x = progress_normalized * transitionDistance;

	int tmp;
	GLfloat angleIndex;
	rotation.z = 1;
	if(progress < 0.3 || progress > 0.65)
		rotation.angle = -45.0f;
	else
	{
		tmp = progress / 0.1;
		angleIndex = progress - tmp * 0.1f;
		rotation.angle = 45.0f - 90.0f / 0.1f * angleIndex;
	}

	glTranslatef(displacement.x, displacement.y, displacement.z);
	glRotatef(rotation.angle, rotation.x, rotation.y, rotation.z);
}


void PerformLeftTransition() 
{
	Displacement displacement;
	ZeroMemory(&displacement, sizeof(Displacement));

	GLfloat progress = (GLfloat)current_time / SCENE_DURATION;
	GLfloat progress_normalized;

	if (progress < 0.1f) {
		progress_normalized = 0;
	}
	else if(progress < 0.65f)
	{
		progress_normalized = (progress - 0.1f) / 0.55f;
	}
	else if(progress >= 0.65f)
	{
		progress_normalized = 1;
	}

	displacement.x = progress_normalized * transitionDistance;

	glTranslatef(displacement.x, displacement.y, displacement.z);
}

void PerformLeftLegsTransition(bool isLeftLeg)
{
	Displacement displacement;
	ZeroMemory(&displacement, sizeof(Displacement));

	GLfloat progress = (GLfloat)current_time / SCENE_DURATION;
	GLfloat progress_normalized;

	if (progress < 0.1f) {
		displacement.x = 0;
	}
	else if(progress < 0.65f)
	{
		progress_normalized = (progress - 0.1f) / 0.55f;
		displacement.x = progress_normalized * transitionDistance;
		glTranslatef(displacement.x, displacement.y, displacement.z);

		int tmp = progress / 0.2f;							//  
		GLfloat displacementIndex = progress - tmp * 0.2f;
		displacement.x = 0.2f / 0.2f * displacementIndex;
		if(isLeftLeg == false)
			displacement.x *= -1;
	}
	else if(progress >= 0.65f)
	{
		progress_normalized = 1;
		displacement.x = progress_normalized * transitionDistance;
	}

	glTranslatef(displacement.x, displacement.y, displacement.z);
}

void PerformRightBodyTransition(bool isTopPart) 
{
	Displacement displacement;
	Rotation rotation;
	ZeroMemory(&displacement, sizeof(Displacement));
	ZeroMemory(&rotation, sizeof(Rotation));

	GLfloat progress = (GLfloat)current_time / SCENE_DURATION;
	GLfloat progress_normalized;

	if (progress < 0.1f) {
		progress_normalized = 0;
	}
	else if(progress < 0.65f)
	{
		progress_normalized = (progress - 0.1f) / 0.55f;
	}
	
	if(progress >= 0.65f)
	{
		progress_normalized = (progress - 0.1f) / 0.55f;
		if(isTopPart)
		{
			displacement.y = (progress_normalized - 1) * 4;
			displacement.x = -transitionDistance - (progress_normalized - 1);
			rotation.angle = progress_normalized * 180 * 1;
			
		}
		else
		{
			displacement.y = -(progress_normalized - 1) * 1.5;
			displacement.x = -transitionDistance + progress_normalized - 1;
			rotation.angle = progress_normalized * 180 * 4;
		}
		rotation.z = 1;
	}
	else
	{
		displacement.x = -progress_normalized * transitionDistance;
	}
	

	glTranslatef(displacement.x, displacement.y, displacement.z);
	glRotatef(rotation.angle, rotation.x, rotation.y, rotation.z);
}

void PerformRightHornsTransition(bool isLeftHorn) 
{
	Displacement displacement;
	Rotation rotation;
	ZeroMemory(&displacement, sizeof(Displacement));
	ZeroMemory(&rotation, sizeof(Rotation));

	GLfloat progress = (GLfloat)current_time / SCENE_DURATION;
	GLfloat progress_normalized;

	if (progress < 0.1f) {
		progress_normalized = 0;
	}
	else if(progress < 0.65f)
	{
		progress_normalized = (progress - 0.1f) / 0.55f;
	}

	if(progress >= 0.65f)
	{
		progress_normalized = (progress - 0.1f) / 0.55f;
		if(isLeftHorn)
		{
			displacement.y = (progress_normalized - 1) * 3;
			displacement.x = -transitionDistance - (progress_normalized - 1)*2;
			rotation.angle = progress_normalized * 180 * 1;

		}
		else
		{
			displacement.y = -(progress_normalized - 1) * 1.5;
			displacement.x = -transitionDistance + progress_normalized - 1;
			rotation.angle = progress_normalized * 180 * 4;
		}
		rotation.z = 1;
	}
	else
	{
		displacement.x = -progress_normalized * transitionDistance;
	}


	glTranslatef(displacement.x, displacement.y, displacement.z);
	glRotatef(rotation.angle, rotation.x, rotation.y, rotation.z);
}

void PerformRightHeadTransition() 
{
	Displacement displacement;
	Rotation rotation;
	ZeroMemory(&displacement, sizeof(Displacement));
	ZeroMemory(&rotation, sizeof(Rotation));

	GLfloat progress = (GLfloat)current_time / SCENE_DURATION;
	GLfloat progress_normalized;

	if (progress < 0.1f) {
		progress_normalized = 0;
	}
	else if(progress < 0.65f)
	{
		progress_normalized = (progress - 0.1f) / 0.55f;
	}

	if(progress >= 0.65f)
	{
		progress_normalized = (progress - 0.1f) / 0.55f;
		displacement.y = (progress_normalized - 1) * 1.2;
		displacement.x = -transitionDistance + (progress_normalized - 1) * 3;

		rotation.angle = progress_normalized * 180 * 2.5;
		rotation.z = 1;
	}
	else
	{
		displacement.x = -progress_normalized * transitionDistance;
	}

	glTranslatef(displacement.x, displacement.y, displacement.z);
	glRotatef(rotation.angle, rotation.x, rotation.y, rotation.z);
}

void PerformRightHandTransitionAndRotation() 
{
	Displacement displacement;
	Rotation rotation;
	ZeroMemory(&displacement, sizeof(Displacement));
	ZeroMemory(&rotation, sizeof(Rotation));

	rotation.z = 1;

	GLfloat progress = (GLfloat)current_time / SCENE_DURATION;
	GLfloat progress_normalized;

	if (progress < 0.1f) {
		progress_normalized = 0;
	}
	else if(progress < 0.65f)
	{
		progress_normalized = (progress - 0.1f) / 0.55f;
	}
	
	if(progress >= 0.65f)
	{
		progress_normalized = (progress - 0.1f) / 0.55f;
		displacement.y = (progress_normalized - 1) * 1.2;
		displacement.x = -transitionDistance - (progress_normalized - 1) * 3;

		rotation.angle = progress_normalized * 180 * 2.5;
	}
	else
	{
		displacement.x = -progress_normalized * transitionDistance;
	}

	if(progress < 0.3)
		rotation.angle = 45.0f;
	else if(progress < 0.65)
	{
		int tmp = progress / 0.1;
		GLfloat angleIndex = progress - tmp * 0.1f;
		rotation.angle = 45.0f - 90.0f / 0.1f * angleIndex;
	}
	else
	{
		progress_normalized = (progress - 0.1f) / 0.55f;
		rotation.angle = progress_normalized * 180 * 2.5;
	}

	glTranslatef(displacement.x, displacement.y, displacement.z);
	glRotatef(rotation.angle, rotation.x, rotation.y, rotation.z);
}

void PerformRightLegsTransition(bool isLeftLeg)
{
	Displacement displacement;
	Rotation rotation;
	ZeroMemory(&displacement, sizeof(Displacement));
	ZeroMemory(&rotation, sizeof(Rotation));

	GLfloat progress = (GLfloat)current_time / SCENE_DURATION;
	GLfloat progress_normalized;

	if (progress < 0.1f) {
		displacement.x = 0;
	}
	else if(progress < 0.65f)
	{
		progress_normalized = (progress - 0.1f) / 0.55f;
		displacement.x = -progress_normalized * transitionDistance;
		glTranslatef(displacement.x, displacement.y, displacement.z);

		int tmp = progress / 0.2f;							//  
		GLfloat displacementIndex = progress - tmp * 0.2f;
		displacement.x = 0.2f / 0.2f * displacementIndex;
		if(isLeftLeg == false)
			displacement.x *= -1;
	}
	else if(progress >= 0.65f)
	{
		progress_normalized = (progress - 0.1f) / 0.55f;
		displacement.y = -(progress_normalized - 1) * 2;
		if(isLeftLeg)
		{
			displacement.x = -transitionDistance - (progress_normalized - 1) * 2;
			rotation.angle = progress_normalized * 360 * 5;
		}
		else 
		{
			displacement.x = -transitionDistance + (progress_normalized - 1) * 1.5;
			rotation.angle = progress_normalized * 180 * 2;
		}

		rotation.z = 1;
	}

	glTranslatef(displacement.x, displacement.y, displacement.z);
	glRotatef(rotation.angle, rotation.x, rotation.y, rotation.z);
}

void display()
{
	//glClearColor(0.0f, 0.0f, 0.0f, 0.0f); // Set background color to black and opaque
	GLclampf curColor = (float)current_time / (float)SCENE_DURATION;	
	glClearColor(curColor, curColor, curColor, 0.0f); // Set background color to black and opaque
	glClear(GL_COLOR_BUFFER_BIT);         // Clear the color buffer (background)

	// Robo left part X -  
	glPushMatrix();
	glTranslatef(-1.4f, 0.0f, 0.0f);  //   
	glTranslatef(-0.15f, 0.26f, 0.0f);  //   .   
	
	PerformLeftTransition();
	glRotatef(45, 0.0f, 0.0f, 1.0f);
	glBegin(GL_QUADS);
		glColor3f(0.3, 0.2, 0.6);
		glVertex2f( 0.0f,  0.0f);
		glColor3f(1.0, 1.0, 1.0);
		glVertex2f( 0.0f,  -0.08f);
		glVertex2f(-0.3f,  -0.08f);
		glVertex2f(-0.3f,  0.0f);	
	glEnd();
	glPopMatrix();

	// Robo left part 1 - 
	glPushMatrix();
	glTranslatef(-1.4f, 0.0f, 0.0f);  //   
	PerformLeftTransition();
	glBegin(GL_QUADS);
		glColor3f(0.42, 0.14, 0.714);
		glVertex2f(-0.2f, -0.3f);
		glVertex2f(-0.2f,  0.3f);
		glVertex2f( 0.2f,  0.3f);
		glVertex2f( 0.2f, -0.3f);	
	glEnd();
	glPopMatrix();
	
	// Robo left part 2 - 
	glPushMatrix();
	glTranslatef(-1.4f, 0.0f, 0.0f);  //   
	PerformLeftTransition();
	glBegin(GL_QUADS);
		glColor3f(0.92, 0.04, 0.54);
		glVertex2f(-0.1f,  0.5f);
		glVertex2f( 0.1f,  0.5f);
		glVertex2f( 0.1f,  0.3f);
		glVertex2f(-0.1f,  0.3f);	
	glEnd();
	glPopMatrix();

	// Robo left part 3 - 
	glPushMatrix();
	glTranslatef(-1.4f, 0.0f, 0.0f);  //   

	glTranslatef(0.15f, 0.26f, 0.0f);  //   .   
	PerformLeftHandTransitionAndRotation();
	glBegin(GL_QUADS);
		glColor3f(0.42, 0.14, 0.214);
		glVertex2f( 0.0f,  0.0f);
		glVertex2f( 0.0f,  -0.08f);
		glColor3f(1.0, 1.0, 0.8);
		glVertex2f( 0.3f,  -0.08f);
		glVertex2f( 0.3f,  0.0f);	
	glEnd();
	// 
	glBegin(GL_POLYGON);	
		glColor3f(1.0, 0.0, 0.0);
		glVertex2f(0.3f,  -0.03f);
		glVertex2f(0.6f,  -0.03f);
		glVertex2f(0.8f,  -0.04f);
		glVertex2f(0.6f,  -0.05f);
		glVertex2f(0.3f,  -0.05f);	
	glEnd();
	glPopMatrix();

	// Robo left part 4 -  
	glPushMatrix();
	glTranslatef(-1.4f, 0.0f, 0.0f);  //   
	glTranslatef(-0.1f, -0.5f, 0.0f);  //  
	PerformLeftLegsTransition(true);
	glBegin(GL_QUADS);
		glColor3f(0.52, 0.14, 0.04);
		glVertex2f(-0.05f,  0.2f);
		glVertex2f(-0.05f, -0.2f);
		glVertex2f( 0.05f,  -0.2f);
		glVertex2f( 0.05f,  0.2f);	
	glEnd();
	glPopMatrix();

	// Robo left part 5 -  
	glPushMatrix();
	glTranslatef(-1.4f, 0.0f, 0.0f);  //   
	glTranslatef(0.1f, -0.5f, 0.0f);  //  
	PerformLeftLegsTransition(false);
	glBegin(GL_QUADS);
		glColor3f(0.52, 0.14, 0.04);
		glVertex2f(-0.05f,  0.2f);
		glVertex2f(-0.05f, -0.2f);
		glVertex2f( 0.05f, -0.2f);
		glVertex2f( 0.05f,  0.2f);	
	glEnd();
	glPopMatrix();

	// Robo left part 6 - 
	glPushMatrix();
	PerformLeftTransition();
	glTranslatef(-1.4f, 0.0f, 0.0f);  //   
	glTranslatef(0.0f, 0.5f, 0.0f);  //    
	
	glBegin(GL_TRIANGLES);
		glColor3f(0.52, 0.24, 0.54);
		glVertex2f(-0.02f, 0.0f);
		glColor3f(0.10, 0.84, 0.34);
		glVertex2f(-0.08f, 0.0f);
		glColor3f(0.92, 0.54, 0.04);
		glVertex2f(-0.05f, 0.09f);
	glEnd();
	glBegin(GL_TRIANGLES);
		glColor3f(0.52, 0.24, 0.54);
		glVertex2f(0.02f, 0.0f);
		glColor3f(0.10, 0.84, 0.34);
		glVertex2f(0.08f, 0.0f);
		glColor3f(0.92, 0.54, 0.04);
		glVertex2f(0.05f, 0.09f);
	glEnd();
	glPopMatrix();
	//-----------------------------------------------------------
	//-----------------------------------------------------------

	// Robo right part X -  
	glPushMatrix();
	glTranslatef( 1.4f, 0.0f, 0.0f);  //   
	glTranslatef( 0.15f, 0.26f, 0.0f);  //   .   

	PerformRightBodyTransition(true);
	glRotatef(-45, 0.0f, 0.0f, 1.0f);
	glBegin(GL_QUADS);
		glColor3f(0.42, 0.14, 0.214);
		glVertex2f( 0.0f,  0.0f);
		glColor3f(1.0, 1.0, 0.8);
		glVertex2f( 0.0f,  -0.08f);
		glVertex2f(0.3f,  -0.08f);
		glVertex2f(0.3f,  0.0f);	
	glEnd();
	glPopMatrix();

	// Robo right part 1 - 
	// 1.1  
	glPushMatrix();
	glTranslatef(1.4f, 0.0f, 0.0f);  //   
	PerformRightBodyTransition(true);
	glBegin(GL_QUADS);
		glColor3f(0.0, 1.0, 0.0);
		glVertex2f(-0.2f,  0.2f);
		glVertex2f(-0.2f,  0.3f);
		glVertex2f( 0.2f,  0.3f);
		glVertex2f( 0.2f,  0.1f);	
	glEnd();
	glPopMatrix();
	// 1.2  
	glPushMatrix();
	glTranslatef(1.4f, 0.0f, 0.0f);  //   
	PerformRightBodyTransition(false);
	glBegin(GL_QUADS);
		glColor3f(0.0, 1.0, 0.0);
		glVertex2f(-0.2f, -0.3f);
		glVertex2f(-0.2f,  0.2f);
		glVertex2f( 0.2f,  0.1f);
		glVertex2f( 0.2f, -0.3f);	
	glEnd();
	glPopMatrix();

	// Robo right part 2 - 
	glPushMatrix();
	glTranslatef(1.4f, 0.0f, 0.0f);  //   
	glTranslatef(0.0f, 0.4f, 0.0f);  //  
	PerformRightHeadTransition();
	glBegin(GL_QUADS);
		glColor3f(0.8, 0.1, 0.2);
		glVertex2f(-0.1f,  0.1f);
		glVertex2f( 0.1f,  0.1f);
		glColor3f(0.8, 0.3, 0.1);
		glVertex2f( 0.1f,  -0.1f);
		glVertex2f(-0.1f,  -0.1f);	
	glEnd();
	glPopMatrix();

	//  Robo right part 3 - 
	glPushMatrix();
	glTranslatef(1.4f, 0.0f, 0.0f);  //   

	glTranslatef(-0.15f, 0.26f, 0.0f);  //   .   
	//glRotatef(45, 0, 0, 1);
	PerformRightHandTransitionAndRotation();
	glBegin(GL_QUADS);	
		  glColor3f(0.3, 0.2, 0.6);
		glVertex2f( 0.0f,  0.0f);
		  glColor3f(1.0, 1.0, 1.0);
		glVertex2f( 0.0f,  -0.08f);
		glVertex2f(-0.3f,  -0.08f);
		glVertex2f(-0.3f,  0.0f);	
	glEnd();	
	// 
	glBegin(GL_POLYGON);	
		  glColor3f(1.0, 0.0, 0.0);
		glVertex2f(-0.3f,  -0.03f);
		glVertex2f(-0.6f,  -0.03f);
		glVertex2f(-0.8f,  -0.04f);
		glVertex2f(-0.6f,  -0.05f);
		glVertex2f(-0.3f,  -0.05f);	
	glEnd();	
	glPopMatrix();

	// Robo right part 4 -  
	glPushMatrix();
	glTranslatef(1.4f, 0.0f, 0.0f);  //   
	glTranslatef(-0.1f, -0.5f, 0.0f);  //  
	PerformRightLegsTransition(true);
	glBegin(GL_QUADS);
	glColor3f(0.22, 0.24, 0.64);
	glVertex2f(-0.05f,  0.2f);
	glVertex2f(-0.05f, -0.2f);
	glVertex2f( 0.05f,  -0.2f);
	glVertex2f( 0.05f,  0.2f);	
	glEnd();
	glPopMatrix();

	// Robo right part 5 -  
	glPushMatrix();
	glTranslatef(1.4f, 0.0f, 0.0f);  //   
	glTranslatef(0.1f, -0.5f, 0.0f);  //  
	PerformRightLegsTransition(false);
	glBegin(GL_QUADS);
	glColor3f(0.22, 0.24, 0.64);
	glVertex2f(-0.05f,  0.2f);
	glVertex2f(-0.05f, -0.2f);
	glVertex2f( 0.05f,  -0.2f);
	glVertex2f( 0.05f,  0.2f);	
	glEnd();
	glPopMatrix();

	// Robo right part 6 - 
	glPushMatrix();	
	glTranslatef(1.4f, 0.0f, 0.0f);  //   
	glTranslatef(0.0f, 0.5f, 0.0f);  //    
	PerformRightHornsTransition(true);
	glBegin(GL_TRIANGLES);
		glColor3f(0.52, 0.24, 0.54);
		glVertex2f(-0.02f, 0.0f);
		glColor3f(0.10, 0.84, 0.34);
		glVertex2f(-0.08f, 0.0f);
		glColor3f(0.92, 0.54, 0.04);
		glVertex2f(-0.05f, 0.09f);
	glEnd();
	glPopMatrix();

	glPushMatrix();
	glTranslatef(1.4f, 0.0f, 0.0f);  //   
	glTranslatef(0.0f, 0.5f, 0.0f);  //    
	PerformRightHornsTransition(false);
	glBegin(GL_TRIANGLES);
		glColor3f(0.52, 0.24, 0.54);
		glVertex2f(0.02f, 0.0f);
		glColor3f(0.10, 0.84, 0.34);
		glVertex2f(0.08f, 0.0f);
		glColor3f(0.92, 0.54, 0.04);
		glVertex2f(0.05f, 0.09f);
	glEnd();
	glPopMatrix();

	glutSwapBuffers();
}

void keyboardFunc(unsigned char key, int x, int y) 
{
	if(key == 27) // Esc
		exit(0);
}

void reshapeFunc(int width, int height) 
{
	GLfloat aspect = (GLfloat)width / (GLfloat)(height == 0 ? 1 : height);

	glViewport(0, 0, width, height);

	glMatrixMode(GL_PROJECTION);  // To operate on the Projection matrix
	glLoadIdentity();             // Reset the projection matrix
	if (width >= height) {
		gluOrtho2D(-1.0f * aspect, 1.0f * aspect, -1.0f, 1.0f);
	}
	else {
		gluOrtho2D(-1.0f, 1.0f, -1.0f / aspect, 1.0f / aspect);
	}
}

void timerFunc(int i) 
{
	static int time_delta = 1;

	current_time = i;

	if (i == SCENE_DURATION || i == -1) 
	{
		time_delta = -time_delta;
		//i = -1;
	}

	glutPostRedisplay();
	glutTimerFunc(TIME_INTERVAL_MS, timerFunc, i + time_delta);
}

int main(int argc, char **argv) 
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
	glutInitWindowSize(SCREEN_WIDTH, SCREEN_HEIGHT);
	glutCreateWindow("Robots");
	glutInitWindowPosition(0, 0);
	glutDisplayFunc(display);
	glutReshapeFunc(reshapeFunc);
	glutKeyboardFunc(keyboardFunc);
	glutTimerFunc(TIME_INTERVAL_MS, timerFunc, 0);
	glutFullScreen();
	glutMainLoop();
	return 0;
}
